@@ -13,12 +13,15 @@ import struct
 import re
 import logging
 from .core import (Message, FailReply, DEFAULT_KATCP_MAJOR,
-                   SEC_TS_KATCP_MAJOR, SEC_TO_MS_FAC, MS_TO_SEC_FAC)
+                   SEC_TS_KATCP_MAJOR, SEC_TO_MS_FAC, MS_TO_SEC_FAC,
+                   convert_method_name)
 
 logger = logging.getLogger(__name__)
 
 # KATCP Type Classes
 #
+# TODO (NM) Consider a 'validate_only' flag where a type is only validated, but
+# not encoded when sent through to a handler.
 
 class KatcpType(object):
     """Class representing a KATCP type.
@@ -209,7 +212,8 @@ class Bool(KatcpType):
 
     def decode(self, value, major):
         if value not in ("0", "1"):
-            raise ValueError("Boolean value must be 0 or 1.")
+            raise ValueError("Boolean value must be '0' or '1' but is '%s'."
+                % (value,))
         return value == "1"
 
 
@@ -765,7 +769,7 @@ def return_reply(*types, **options):
         if not handler.__name__.startswith("request_"):
             raise ValueError("This decorator can only be used on a katcp"
                              " request handler.")
-        msgname = handler.__name__[8:].replace("_", "-")
+        msgname = convert_method_name('request_', handler.__name__)
 
         def raw_handler(self, *args):
             reply_args = handler(self, *args)
