@@ -15,6 +15,7 @@ import heapq
 import Queue
 import os
 
+from functools import partial
 from .core import Message, Sensor, ExcepthookThread, SEC_TO_MS_FAC, MS_TO_SEC_FAC
 
 
@@ -48,7 +49,7 @@ class SampleStrategy(object):
     """
 
     # Sampling strategy constants
-    NONE, AUTO, PERIOD, EVENT, DIFFERENTIAL, EVENT_RATE = range(6)
+    NONE, AUTO, PERIOD, EVENT, DIFFERENTIAL, EVENT_RATE, DIFFERENTIAL_RATE = range(7)
 
     ## @brief Mapping from strategy constant to strategy name.
     SAMPLING_LOOKUP = {
@@ -58,6 +59,7 @@ class SampleStrategy(object):
         EVENT: "event",
         DIFFERENTIAL: "differential",
         EVENT_RATE: "event-rate",
+        DIFFERENTIAL_RATE: "differential-rate",
     }
 
     # SAMPLING_LOOKUP not found by pylint
@@ -111,6 +113,8 @@ class SampleStrategy(object):
             return SamplePeriod(inform_callback, sensor, *params)
         elif strategyType == cls.EVENT_RATE:
             return SampleEventRate(inform_callback, sensor, *params)
+        elif strategyType == cls.DIFFERENTIAL_RATE:
+            return SampleDifferentialRate(inform_callback, sensor, *params)
 
     def update(self, sensor):
         """Callback used by the sensor's notify method.
@@ -155,6 +159,22 @@ class SampleStrategy(object):
         timestamp, status, value = self._sensor.read_formatted()
         self._inform_callback(self._sensor.name, timestamp, status, value)
 
+    def set_new_period_callback(self, new_period_callback):
+        """Set a function that will be called whenever a new period callback needs to be set
+
+        Arguments
+        ---------
+
+        new_period_callback(strategy, next_time) -- Callback function that takes a strategy
+            to be re-periodeded as parameter.
+
+        It is epxected that the reactor will not remove other periodic entries
+        that were previously created, so the period() update logic should
+        suppress spurious updates.
+        """
+
+        self._new_period_callback = partial(new_period_callback, self)
+
     def get_sampling(self):
         """Return the Strategy constant for this sampling strategy.
 
@@ -278,7 +298,6 @@ class SampleDifferential(SampleStrategy):
         self.update(self._sensor)
         super(SampleDifferential, self).attach()
 
-
 class SamplePeriod(SampleStrategy):
     """Periodic sampling strategy.
 
@@ -308,9 +327,9 @@ class SampleEventRate(SampleStrategy):
     """Event rate sampling strategy.
 
     Report the sensor value whenever it changes or if more than
-    longest_period milliseconds have passed since the last reported
+    longest_period seconds have passed since the last reported
     update. However, do not report the value if less than
-    shortest_period milliseconds have passed since the last reported
+    shortest_period seconds have passed since the last reported
     update.
     """
 
@@ -320,12 +339,13 @@ class SampleEventRate(SampleStrategy):
             raise ValueError("The 'event-rate' strategy takes two parameters.")
         shortest_period = float(params[0])
         longest_period = float(params[1])
-        self._lastStatus = None
-        self._lastValue = None
+        # Last sensor status / value, used to determine if it has changed
+        self._last_sv = (None, None)
+        self._next_periodic = 1e99
 
         if not 0 <= shortest_period <= longest_period:
-            raise ValueError("The longest and shortest periods must"
-                             " satisfy 0 <= shorest_period <= longest_period")
+            raise ValueError("The shortest and longest periods must"
+                             " satisfy 0 <= shortest_period <= longest_period")
         self._shortest_period = shortest_period
         self._longest_period = longest_period
         # don't send updates until timestamp _not_before
@@ -337,22 +357,32 @@ class SampleEventRate(SampleStrategy):
     def update(self, sensor, now=None):
         if now is None:
             now = self._time()
+
+        _, status, value = sensor.read()
+        last_s, last_v = self._last_sv
+        sensor_changed = status != last_s or value != last_v
+
         if now < self._not_before:
+            if self._next_periodic != self._not_before and sensor_changed:
+                # If you get an AttributeError here it's because
+                # set_new_period_callback() should have been called
+                self._next_periodic = self._not_before
+                self._new_period_callback(self._not_before)
             return
 
         past_longest = now >= self._not_before + self._not_after_delta
-        _, status, value = sensor.read()
-        sensor_changed = status != self._lastStatus or value != self._lastValue
 
         if past_longest or sensor_changed:
             self._not_before = now + self._shortest_period
-            self._lastStatus = status
-            self._lastValue = value
+            self._last_sv = (status, value)
             self.inform()
 
     def periodic(self, timestamp):
         self.update(self._sensor, now=timestamp)
-        return self._not_before + self._not_after_delta
+        np = self._next_periodic = self._not_before + self._not_after_delta
+        # Don't bother with a next update if it is going to be beyond the
+        # approximate age of the universe
+        return np if np < 4.32329886e17 else None
 
     def get_sampling(self):
         return SampleStrategy.EVENT_RATE
@@ -388,6 +418,85 @@ class SampleEvent(SampleEventRate):
     def get_sampling(self):
         return SampleStrategy.EVENT
 
+class SampleDifferentialRate(SampleStrategy):
+    """Event rate sampling strategy.
+
+    Report the value whenever it changes by more than `difference` from the last
+    reported value or if more than longest-period seconds have passed since the
+    last reported update. However, do not report the value until shortest-period
+    seconds have passed since the last reported update. The behaviour if
+    shortest-period is greater than longest-period is undefined. May only be
+    implemented for float and integer sensors.
+    """
+
+    def __init__(self, inform_callback, sensor, *params):
+        SampleStrategy.__init__(self, inform_callback, sensor, *params)
+        if len(params) != 3:
+            raise ValueError("The 'differential-rate' strategy takes three parameters.")
+        difference = params[0]
+        if sensor.stype ==  'integer':
+            difference = int(difference)
+        elif sensor.stype == 'float':
+            difference = float(difference)
+        else:
+            raise ValueError('The differential-rate strategy can only be defined '
+                             'for integer or float sensors')
+        shortest_period = float(params[1])
+        longest_period = float(params[2])
+        # Last sensor status / value, used to determine if it has changed
+        self._last_sv = (None, None)
+        self._next_periodic = 1e99
+
+        if not 0 <= shortest_period <= longest_period:
+            raise ValueError("The shortest and longest periods must"
+                             " satisfy 0 <= shortest_period <= longest_period")
+        self._shortest_period = shortest_period
+        self._longest_period = longest_period
+        self.difference = difference
+        # don't send updates until timestamp _not_before
+        self._not_before = 0
+        # time between _not_before and next required update
+        self._not_after_delta = (self._longest_period - self._shortest_period)
+        self._time = time.time
+
+    def update(self, sensor, now=None):
+        if now is None:
+            now = self._time()
+
+        _, status, value = sensor.read()
+        last_s, last_v = self._last_sv
+        sensor_changed = (status != last_s or
+                          abs(value - last_v) > self.difference)
+
+        if now < self._not_before:
+            if self._next_periodic != self._not_before and sensor_changed:
+                # If you get an AttributeError here it's because
+                # set_new_period_callback() should have been called
+                self._next_periodic = self._not_before
+                self._new_period_callback(self._not_before)
+            return
+
+        past_longest = now >= self._not_before + self._not_after_delta
+
+        if past_longest or sensor_changed:
+            self._not_before = now + self._shortest_period
+            self._last_sv = (status, value)
+            self.inform()
+
+    def periodic(self, timestamp):
+        self.update(self._sensor, now=timestamp)
+        np = self._next_periodic = self._not_before + self._not_after_delta
+        # Don't bother with a next update if it is going to be beyond the
+        # approximate age of the universe
+        return np if np < 4.32329886e17 else None
+
+    def get_sampling(self):
+        return SampleStrategy.DIFFERENTIAL_RATE
+
+    def attach(self):
+        self.update(self._sensor)
+        super(SampleDifferentialRate, self).attach()
+
 class SampleReactor(ExcepthookThread):
     """SampleReactor manages sampling strategies.
 
@@ -407,6 +516,7 @@ class SampleReactor(ExcepthookThread):
         self._wakeEvent = threading.Event()
         self._heap = []
         self._removal_events = Queue.Queue()
+        self._adding_events = Queue.Queue()
         self._logger = logger
         # set daemon True so that the app can stop even if the thread
         # is running
@@ -426,11 +536,18 @@ class SampleReactor(ExcepthookThread):
             The sampling strategy to add to the reactor.
         """
         self._strategies.add(strategy)
+        strategy.set_new_period_callback(self.adjust_strategy_update_time)
         strategy.attach()
 
         next_time = strategy.periodic(time.time())
         if next_time is not None:
-            heapq.heappush(self._heap, (next_time, strategy))
+            self._adding_events.put((next_time, strategy))
+            self._wakeEvent.set()
+
+    def adjust_strategy_update_time(self, strategy, next_time):
+        """Called by a strategy if it needs to have a periodic update time adjusted"""
+        if next_time is not None:
+            self._adding_events.put((next_time, strategy))
             self._wakeEvent.set()
 
     def remove_strategy(self, strategy):
@@ -471,6 +588,12 @@ class SampleReactor(ExcepthookThread):
         self._heapify = heapq.heapify
 
         while not self._stopEvent.isSet():
+            # Push new events into the heap
+            while True:
+                try:
+                    _push(heap, self._adding_events.get(block=False))
+                except Queue.Empty:
+                    break
             self._remove_dead_events()
             wake.clear()
             if heap:
@@ -485,7 +608,8 @@ class SampleReactor(ExcepthookThread):
 
                 try:
                     next_time = strategy.periodic(next_time)
-                    _push(heap, (next_time, strategy))
+                    if next_time is not None:
+                        _push(heap, (next_time, strategy))
                 except Exception, e:
                     self._logger.exception(e)
                     # push ten seconds into the future and hope whatever was
