@@ -24,12 +24,29 @@ from .core import (DeviceMetaclass, ExcepthookThread, Message, MessageParser,
 from .sampling import SampleReactor, SampleStrategy, SampleNone
 from .sampling import format_inform_v5, format_inform_v4
 from .core import (SEC_TO_MS_FAC, MS_TO_SEC_FAC, SEC_TS_KATCP_MAJOR,
-                   VERSION_CONNECT_KATCP_MAJOR, DEFAULT_KATCP_MAJOR)
+                   VERSION_CONNECT_KATCP_MAJOR, DEFAULT_KATCP_MAJOR,
+                   convert_method_name)
 from .version import VERSION, VERSION_STR
 from .kattypes import (request, return_reply)
 
 log = logging.getLogger("katcp")
 
+BASE_REQUESTS = frozenset(['client-list',
+                           'halt',
+                           'help',
+                           'log-level',
+                           'new-command',
+                           'raise-exception',
+                           'raise-fail',
+                           'restart',
+                           'sensor-list',
+                           'sensor-sampling-clear',
+                           'sensor-value',
+                           'version-list',
+                           'watchdog',
+                           'sensor-sampling',])
+"List of basic KATCP request that a minimal device server should implement"
+
 def construct_name_filter(pattern):
     """Return a function for filtering sensor names based on a pattern.
 
@@ -320,7 +337,7 @@ class DeviceServerBase(object):
             The message to process.
         """
         # log messages received so that no one else has to
-        self._logger.debug(msg)
+        self._logger.debug(str(msg))
 
         if msg.mtype == msg.REQUEST:
             self.handle_request(client_conn, msg)
@@ -516,7 +533,7 @@ class DeviceServerBase(object):
         if isinstance(connection, ClientRequestConnection):
             self._logger.warn(
                 'Deprecation warning: do not use self.inform() '
-                'within a reply handler context -- use req.reply()\n'
+                'within a reply handler context -- use req.inform()\n'
                 'Traceback:\n %s', "".join(traceback.format_stack() ))
             # Get the underlying ClientConnectionTCP instance
             connection = connection.client_connection
@@ -1783,6 +1800,8 @@ class DeviceLogger(object):
             The value to set the logging level to.
         """
         self._log_level = level
+        if self._python_logger:
+            self._python_logger.setLevel(level)
 
     def set_log_level_by_name(self, level_name):
         """Set the logging level using a level name.
@@ -1792,7 +1811,7 @@ class DeviceLogger(object):
         level_name : str
             The name of the logging level.
         """
-        self._log_level = self.level_from_name(level_name)
+        self.set_log_level(self.level_from_name(level_name))
 
     def log(self, level, msg, *args, **kwargs):
         """Log a message and inform all clients.
@@ -1872,3 +1891,4 @@ class DeviceLogger(object):
                     "warn": logging.WARN,
                     "error": logging.ERROR,
                     "fatal": logging.FATAL}[level], log_string)
+
