@@ -84,11 +84,13 @@ class MessageRecorder(object):
     """
         Parameters
         ----------
-        whitelist : list or tuple
+        msg_types : container supporting `in` operator
+            Record only message of these types (eg. [Message.REQUEST])
+        whitelist : container supporting `in` operator
             Record only messages matching these names.  If the list is empty,
             all received messages will be saved except any specified in the
             blacklist.
-        blacklist : list or tuple
+        blacklist : container supporting `in` operator
             Ignore messages matching these names.  If any names appear both in
             the whitelist and the blacklist, the whitelist will take
             precedence.
@@ -116,7 +118,7 @@ class MessageRecorder(object):
 
         Parameters
         ==========
-        number -- Minumum number of messages to return, else wait
+        min_number -- Minumum number of messages to return, else wait
         timeout: int seconds or None -- Don't wait longer than this
         """
         self.wait_number(min_number, timeout)
@@ -1287,7 +1289,7 @@ def start_thread_with_cleanup(
     """Start thread_object and add cleanup functions to test_instance
 
     thread_object.start() is called to start the thread, or thread_object.start(
-    timeout=start_timeout if the starte_timeout parameter is not None.
+    timeout=start_timeout if the start_timeout parameter is not None.
 
     thread_object.join(timeout=timeout) and thread_object.stop() is added to the
     test instance cleanup. Parameter timeout defaults to 1.
@@ -1299,18 +1301,54 @@ def start_thread_with_cleanup(
     test_instance.addCleanup(thread_object.join, timeout=timeout)
     test_instance.addCleanup(thread_object.stop)
 
+import threading
+from peak.util.proxies import ObjectWrapper
+
+
+class AtomicIaddCallback(ObjectWrapper):
+    # Attributes must be in the class definition, or else they will be proxied to
+    # __subject__
+    _mutex = None
+    _callback = None
+
+    def __init__(self, ob, callback=lambda x: x, mutex=None):
+        self._callback = callback
+        self._mutex = mutex if mutex else threading.RLock()
+        super(AtomicIaddCallback, self).__init__(ob)
+
+    def __iadd__(self, other):
+        with self._mutex:
+            self.__subject__ += other
+            val = self.__subject__
+        self._callback(val)
+        return self
+
 class WaitingMock(mock.Mock):
     def __init__(self, *args, **kwargs):
         super(WaitingMock, self).__init__(*args, **kwargs)
-        self._wait_call_count_event = threading.Event()
-        self._num_calls_waiting_for = None
+        self._counted_queue = Queue.Queue(maxsize=1)
+        # Replace the underlying value for self.call_count with a proxied int that uses a
+        # threading.RLock to allow atomic incrementation in case multiple threads are
+        # calling the mock, and does a callback as soon as the value is updated. This is
+        # needed in case the side_effect function is blocking. The original
+        # mock.CallableMixin._mock_call() updates the call_count before calling
+        # side_effect(). This means we can notify someone waiting on
+        # self.assert_wait_call_count in another thread as soon as the call is made, even
+        # if side_effect blocks
+        self.call_count = AtomicIaddCallback(0, callback=self._call_count_callback)
 
-    def _mock_call(self, *args, **kwargs):
-        retval = super(WaitingMock, self)._mock_call(*args, **kwargs)
-        waiting_for = self._num_calls_waiting_for
-        if waiting_for and self.call_count >= waiting_for:
-            self._wait_call_count_event.set()
-        return retval
+    def _call_count_callback(self, call_count):
+        try:
+            self._counted_queue.put_nowait(call_count)
+        except Queue.Full:
+            pass
+
+    def reset_mock(self):
+        # Re-set call_count as an AtomicIaddCallback instance since the reset_mock()
+        # super-method does self.call_count=0
+        super(WaitingMock, self).reset_mock()
+        self.call_count = AtomicIaddCallback(
+            self.call_count, callback=self._call_count_callback)
 
     def assert_wait_call_count(self, count, timeout=1.):
         """
@@ -1318,12 +1356,18 @@ class WaitingMock(mock.Mock):
 
         Raises AssertionError if the call count is not reached.
         """
-        self._num_calls_waiting_for = count
-        self._wait_call_count_event.clear()
+        t0 = time.time()
+        to_wait = timeout
         if self.call_count >= count:
             return True
-        self._wait_call_count_event.wait(timeout=timeout)
-        assert(self._wait_call_count_event.isSet())
+        while to_wait >= 0 and self.call_count < count:
+            try:
+                self._counted_queue.get(timeout=to_wait)
+            except Queue.Empty:
+                pass
+            to_wait = timeout - (time.time() - t0)
+
+        assert(self.call_count >= count)
 
 def mock_req(req_name, *args, **kwargs):
     """
@@ -1376,9 +1420,40 @@ def mock_req(req_name, *args, **kwargs):
     if not client_conn:
         client_conn = WaitingMock()
 
+    # TODO Consider making the mocks autospecced from the real classes to test somewhat
+    # more strictly.
+
+    # TODO Consider making .reply_msg and .inform_msgs attributes containing the actualy
+    # reply/inform Message objects that would have been sent. May make certain kinds of
+    # test comparisons easier.
+
     req = WaitingMock()
     req.client_connection = client_conn
     req.msg = Message.request(req_name, *args)
     req.make_reply.side_effect = lambda *args: Message.reply_to_request(
         req.msg, *args)
     return req
+
+def handle_mock_req(dev, req):
+    """Instrument a real not-started server.DeviceServer subclass to handle a mock request
+
+    Parameters
+    ----------
+
+    dev : a katcp.server.DeviceServer instance that has not neccesarily been started
+    req : a mock request created with katcp.testutils.mock_req()
+
+    Example
+    -------
+    dev = katcp.server.DeviceServer(...)
+    req = katcp.testutils.mock_request('help')
+    handle_mock_req(dev, req)
+    # All replies / informs can now be asserted on the mock request
+    req.reply.assert_called_once_with('ok')
+    """
+    client_connection = req.client_connection
+    # Hook up client_connection reply so that it logs the reply on the mock request obj
+    client_connection.reply.side_effect = lambda rep_msg, _: req.reply(*rep_msg.arguments)
+    with mock.patch('katcp.server.ClientRequestConnection') as CRC:
+        CRC.return_value = req
+        dev.handle_request(req.client_connection, req.msg)
